#
# Copyright (c) 2008-2015 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
from pyams_skin.interfaces import IContentTitle
from pyams_utils.adapter import adapter_config, ContextRequestViewAdapter

__docformat__ = 'restructuredtext'


# import standard library

# import interfaces
from pyams_portal.interfaces import IPortalTemplateContainer, IPortalTemplate
from pyams_skin.interfaces.viewlet import IToolbarAddingMenu
from pyams_skin.layer import IPyAMSLayer
from pyams_workflow.interfaces import IWorkflowVersions, IWorkflowInfo
from pyams_zmi.layer import IAdminLayer
from z3c.form.interfaces import IDataExtractedEvent
from zope.component.interfaces import ISite

# import packages
from pyams_form.form import AJAXAddForm
from pyams_pagelet.pagelet import pagelet_config
from pyams_portal.template import PortalTemplate, PortalWfTemplate
from pyams_portal.zmi.container import PortalTemplateContainerTable
from pyams_skin.container import delete_container_element
from pyams_skin.viewlet.toolbar import ToolbarMenuItem
from pyams_utils.registry import query_utility
from pyams_utils.url import absolute_url
from pyams_viewlet.viewlet import viewlet_config
from pyams_zmi.form import AdminDialogAddForm
from pyramid.events import subscriber
from pyramid.view import view_config
from z3c.form import field
from zope.interface import Interface, Invalid
from zope.lifecycleevent import ObjectCreatedEvent

from pyams_portal import _


@adapter_config(context=(IPortalTemplate, IPyAMSLayer, Interface), provides=IContentTitle)
class PortalTemplateTitleAdapter(ContextRequestViewAdapter):
    """Portal template title adapter"""

    @property
    def title(self):
        translate = self.request.localizer.translate
        return translate(_("« {0} »  portal template")).format(self.context.name)


#
# Template views
#

@viewlet_config(name='add-portal-template.menu', context=ISite, layer=IAdminLayer,
                view=PortalTemplateContainerTable, manager=IToolbarAddingMenu,
                permission='portal.templates.manage', weight=20)
class PortalTemplateAddMenu(ToolbarMenuItem):
    """Portal template add menu"""

    label = _("Add shared template...")
    label_css_class = 'fa fa-fw fa-columns'
    url = 'add-portal-template.html'
    modal_target = True


@pagelet_config(name='add-portal-template.html', context=ISite, layer=IPyAMSLayer,
                permission='portal.templates.manage')
class PortalTemplateAddForm(AdminDialogAddForm):
    """Portal template add form"""

    title = _("Portal templates")
    legend = _("Add shared template")
    icon_css_class = 'fa fa-fw fa-columns'

    fields = field.Fields(IPortalTemplate)
    ajax_handler = 'add-portal-template.json'
    edit_permission = None

    def create(self, data):
        return PortalTemplate()

    def add(self, template):
        wf_template = PortalWfTemplate()
        self.request.registry.notify(ObjectCreatedEvent(wf_template))
        context = query_utility(IPortalTemplateContainer)
        context[template.name] = wf_template
        IWorkflowVersions(wf_template).add_version(template, None)
        IWorkflowInfo(template).fire_transition('init')

    def nextURL(self):
        return absolute_url(self.context, self.request, 'portal-templates.html')


@subscriber(IDataExtractedEvent, form_selector=PortalTemplateAddForm)
def handle_new_template_data_extraction(event):
    """Handle new template form data extraction"""
    container = query_utility(IPortalTemplateContainer)
    name = event.data.get('name')
    if name in container:
        event.form.widgets.errors += (Invalid(_("Specified name is already used!")),)


@view_config(name='add-portal-template.json', context=ISite, request_type=IPyAMSLayer,
             permission='portal.templates.manage', renderer='json', xhr=True)
class PortalTemplateAJAXAddForm(AJAXAddForm, PortalTemplateAddForm):
    """Portal template add form, AJAX handler"""


@view_config(name='delete-element.json', context=IPortalTemplateContainer, request_type=IPyAMSLayer,
             permission='portal.templates.manage', renderer='json', xhr=True)
def delete_portal_template(request):
    """Delete template from portal"""
    return delete_container_element(request)
